"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const vscode_uri_1 = require("vscode-uri");
const theme_check_common_1 = require("@shopify/theme-check-common");
const diagnostics_1 = require("../../diagnostics");
const documents_1 = require("../../documents");
const SuggestionProvider_1 = require("./SuggestionProvider");
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
(0, vitest_1.describe)('Unit: SuggestionProvider', () => {
    const uri = vscode_uri_1.URI.file('/path/to/file.liquid').toString();
    const contents = `
    {% assign x = 1 %}
    <script src="2.js"></script>
    <script src="3.js"></script>
  `;
    const version = 0;
    const document = vscode_languageserver_textdocument_1.TextDocument.create(uri, 'liquid', version, contents);
    let documentManager;
    let diagnosticsManager;
    let suggestionProvider;
    function makeOffense(checkName, needle, unfixable) {
        const start = contents.indexOf(needle);
        const end = start + needle.length;
        const suggestion = (replaceWith) => ({
            message: `Replace with ${replaceWith}`,
            fix: (corrector) => corrector.replace(start, end, replaceWith),
        });
        return {
            type: theme_check_common_1.SourceCodeType.LiquidHtml,
            check: checkName,
            message: 'Parser blocking script detected',
            absolutePath: '/path/to/file.liquid',
            severity: theme_check_common_1.Severity.ERROR,
            start: { ...document.positionAt(start), index: start },
            end: { ...document.positionAt(end), index: end },
            suggest: unfixable ? undefined : [suggestion('defer'), suggestion('async')],
        };
    }
    (0, vitest_1.beforeEach)(() => {
        documentManager = new documents_1.DocumentManager();
        diagnosticsManager = new diagnostics_1.DiagnosticsManager({ sendDiagnostics: vitest_1.vi.fn() });
        documentManager.open(uri, contents, version);
        suggestionProvider = new SuggestionProvider_1.SuggestionProvider(documentManager, diagnosticsManager);
    });
    (0, vitest_1.describe)('When multiple offenses of various types exist, and the selection includes an offense with at least one more of the same type in the file', () => {
        (0, vitest_1.it)('provides all code actions categories', async () => {
            const offenses = [
                makeOffense('UnusedAssign', '{% assign x = 1 %}'),
                makeOffense('ParserBlockingScript', '<script src="2.js"></script>'),
                makeOffense('ParserBlockingScript', '<script src="3.js"></script>'),
            ];
            diagnosticsManager.set(uri, version, offenses);
            const codeActions = suggestionProvider.codeActions({
                textDocument: { uri },
                range: {
                    start: document.positionAt(contents.indexOf('2.js')),
                    end: document.positionAt(contents.indexOf('2.js')),
                },
                context: { diagnostics: [] },
            });
            (0, vitest_1.expect)(codeActions.length).toBe(2);
            // We're targetting <script src=2.js> (index = 1 in the offense array)
            const anomalyId = 1;
            const [deferSuggestion, asyncSuggestion] = codeActions;
            (0, vitest_1.expect)(deferSuggestion).toEqual({
                title: `Suggestion: Replace with defer`,
                kind: 'quickfix',
                diagnostics: vitest_1.expect.any(Array),
                isPreferred: false,
                command: {
                    title: 'applySuggestion',
                    command: 'themeCheck/applySuggestion',
                    arguments: [uri, version, anomalyId, 0],
                },
            });
            (0, vitest_1.expect)(asyncSuggestion).toEqual({
                title: `Suggestion: Replace with async`,
                kind: 'quickfix',
                diagnostics: vitest_1.expect.any(Array),
                isPreferred: false,
                command: {
                    title: 'applySuggestion',
                    command: 'themeCheck/applySuggestion',
                    arguments: [uri, version, anomalyId, 1],
                },
            });
        });
    });
    (0, vitest_1.it)('does not provide code actions for offenses with no suggestions', async () => {
        const nonFixableOffense = makeOffense('ParserBlockingScript', '<script src="2.js">', true);
        diagnosticsManager.set(uri, version, [nonFixableOffense]);
        const codeActions = suggestionProvider.codeActions({
            textDocument: { uri },
            range: {
                start: document.positionAt(contents.indexOf('2.js')),
                end: document.positionAt(contents.indexOf('2.js')),
            },
            context: { diagnostics: [] },
        });
        (0, vitest_1.expect)(codeActions.length).toBe(0);
    });
    (0, vitest_1.it)('does not provide code actions if selection range does not overlap with the error', async () => {
        diagnosticsManager.set(uri, version, [
            makeOffense('ParserBlockingScript', '<script src="2.js"></script>'),
            makeOffense('ParserBlockingScript', '<script src="3.js"></script>'),
        ]);
        const codeActions = suggestionProvider.codeActions({
            textDocument: { uri },
            range: {
                start: document.positionAt(contents.indexOf('assign x = 1')),
                end: document.positionAt(contents.indexOf('= 1')),
            },
            context: { diagnostics: [] },
        });
        (0, vitest_1.expect)(codeActions.length).toBe(0);
    });
});
//# sourceMappingURL=SuggestionProvider.spec.js.map