"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const vscode_uri_1 = require("vscode-uri");
const theme_check_common_1 = require("@shopify/theme-check-common");
const diagnostics_1 = require("../../diagnostics");
const documents_1 = require("../../documents");
const ApplySuggestionProvider_1 = require("./ApplySuggestionProvider");
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
const offenseToDiagnostic_1 = require("../../diagnostics/offenseToDiagnostic");
(0, vitest_1.describe)('Unit: ApplySuggestionProvider', () => {
    const uri = vscode_uri_1.URI.file('/path/to/file.liquid').toString();
    const contents = `
    {% assign x = 1 %}
    <script src="2.js"></script>
    <script src="3.js"></script>
  `;
    const version = 0;
    const document = vscode_languageserver_textdocument_1.TextDocument.create(uri, 'liquid', version, contents);
    let connection;
    let documentManager;
    let diagnosticsManager;
    let applySuggestionProvider;
    function makeOffense(checkName, needle, unfixable) {
        const start = contents.indexOf(needle);
        const end = start + needle.length;
        const suggestion = (replaceWith) => ({
            message: `Replace with ${replaceWith}`,
            fix: (corrector) => corrector.replace(start, end, replaceWith),
        });
        return {
            type: theme_check_common_1.SourceCodeType.LiquidHtml,
            check: checkName,
            message: 'Parser blocking script detected',
            absolutePath: '/path/to/file.liquid',
            severity: theme_check_common_1.Severity.ERROR,
            start: { ...document.positionAt(start), index: start },
            end: { ...document.positionAt(end), index: end },
            suggest: unfixable ? undefined : [suggestion('defer'), suggestion('async')],
        };
    }
    (0, vitest_1.beforeEach)(() => {
        connection = { sendRequest: vitest_1.vi.fn(), sendDiagnostics: vitest_1.vi.fn() };
        documentManager = new documents_1.DocumentManager();
        diagnosticsManager = new diagnostics_1.DiagnosticsManager(connection);
        documentManager.open(uri, contents, version);
        const capabilities = { hasApplyEditSupport: vitest_1.vi.fn().mockReturnValue(true) };
        applySuggestionProvider = new ApplySuggestionProvider_1.ApplySuggestionProvider(documentManager, diagnosticsManager, capabilities, connection);
    });
    (0, vitest_1.describe)('execute', () => {
        (0, vitest_1.it)('applies the suggestion for the given offense id and suggestion index', async () => {
            const offenses = [
                makeOffense('Offense1', '{% assign x = 1 %}'),
                makeOffense('Offense2', '<script src="2.js"></script>'),
                makeOffense('Offense3', '<script src="3.js"></script>'),
            ];
            diagnosticsManager.set(uri, version, offenses);
            const offenseId = 1; // 2.js
            const suggestionIndex = 1; // async
            await applySuggestionProvider.execute(uri, version, offenseId, suggestionIndex);
            (0, vitest_1.expect)(connection.sendRequest).toHaveBeenCalledWith(vitest_1.expect.anything(), {
                label: 'Apply suggestion: Replace with async',
                edit: {
                    documentChanges: [
                        {
                            textDocument: vitest_1.expect.anything(),
                            edits: [
                                {
                                    newText: 'async',
                                    range: {
                                        start: asLspPosition(offenses[1].start),
                                        end: asLspPosition(offenses[1].end),
                                    },
                                },
                            ],
                        },
                    ],
                },
            });
            // We clean up stale diagnostics when we're done
            (0, vitest_1.expect)(connection.sendDiagnostics).toHaveBeenCalledWith({
                uri: vitest_1.expect.anything(),
                version: vitest_1.expect.any(Number),
                diagnostics: offenses.filter((_, index) => index !== offenseId).map(offenseToDiagnostic_1.offenseToDiagnostic),
            });
        });
        (0, vitest_1.it)('does not apply fixes if the document or diagnostics are not available', async () => {
            const offenses = [
                makeOffense('Offense1', '{% assign x = 1 %}'),
                makeOffense('Offense2', '<script src="2.js"></script>'),
                makeOffense('Offense3', '<script src="3.js"></script>'),
            ];
            diagnosticsManager.set(uri, 0, offenses);
            documentManager.close(uri);
            await applySuggestionProvider.execute(uri, 1, 0, 0);
            (0, vitest_1.expect)(connection.sendRequest).not.toHaveBeenCalled();
        });
    });
});
function omit(obj, key) {
    const clone = { ...obj };
    delete clone[key];
    return clone;
}
function asLspPosition(position) {
    return omit(position, 'index');
}
//# sourceMappingURL=ApplySuggestionProvider.spec.js.map