"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.startServer = void 0;
const vscode_languageserver_1 = require("vscode-languageserver");
const utils_1 = require("../utils");
const diagnostics_1 = require("../diagnostics");
const documents_1 = require("../documents");
const version_1 = require("../version");
const documentLinks_1 = require("../documentLinks");
const codeActions_1 = require("../codeActions");
const completions_1 = require("../completions");
const hover_1 = require("../hover");
const commands_1 = require("../commands");
const ClientCapabilities_1 = require("../ClientCapabilities");
const docset_1 = require("../docset");
const translations_1 = require("../translations");
const defaultLogger = () => { };
/**
 * This code runs in node and the browser, it can't talk to the file system
 * or make requests. Stuff like that should be injected.
 *
 * In browser, theme-check-js wants these things:
 *   - fileExists(path)
 *   - defaultTranslations
 *
 * Which means we gotta provide 'em from here too!
 */
function startServer(connection, { fileExists, fileSize, filesForURI, findRootURI, getDefaultLocaleFactory, getDefaultTranslationsFactory, loadConfig, log = defaultLogger, schemaValidators, themeDocset: remoteThemeDocset, }) {
    const clientCapabilities = new ClientCapabilities_1.ClientCapabilities();
    const documentManager = new documents_1.DocumentManager();
    const diagnosticsManager = new diagnostics_1.DiagnosticsManager(connection);
    const documentLinksProvider = new documentLinks_1.DocumentLinksProvider(documentManager);
    const codeActionsProvider = new codeActions_1.CodeActionsProvider(documentManager, diagnosticsManager);
    const themeDocset = new docset_1.AugmentedThemeDocset(remoteThemeDocset);
    const runChecks = (0, utils_1.debounce)((0, diagnostics_1.makeRunChecks)(documentManager, diagnosticsManager, {
        fileExists,
        fileSize,
        findRootURI,
        getDefaultLocaleFactory,
        getDefaultTranslationsFactory,
        loadConfig,
        themeDocset,
        schemaValidators,
    }), 100);
    const getTranslationsForURI = async (uri) => {
        const rootURI = await findRootURI(uri);
        const theme = documentManager.theme(rootURI);
        return (0, translations_1.useBufferOrInjectedTranslations)(getDefaultTranslationsFactory, theme, rootURI);
    };
    const getSnippetNamesForURI = async (uri) => {
        if (!filesForURI)
            return [];
        const files = await filesForURI(uri);
        return files
            .filter((x) => x.startsWith('snippets'))
            .map((x) => x
            .replace(/\\/g, '/')
            .replace(/^snippets\//, '')
            .replace(/\..*$/, ''));
    };
    const completionsProvider = new completions_1.CompletionsProvider(documentManager, themeDocset, getTranslationsForURI, getSnippetNamesForURI, log);
    const hoverProvider = new hover_1.HoverProvider(documentManager, themeDocset, getTranslationsForURI);
    const executeCommandProvider = new commands_1.ExecuteCommandProvider(documentManager, diagnosticsManager, clientCapabilities, runChecks, connection);
    connection.onInitialize((params) => {
        clientCapabilities.setup(params.capabilities);
        const fileOperationRegistrationOptions = {
            filters: [
                {
                    scheme: 'file',
                    pattern: {
                        glob: '**/*.{liquid,json}',
                    },
                },
            ],
        };
        const result = {
            capabilities: {
                textDocumentSync: {
                    change: vscode_languageserver_1.TextDocumentSyncKind.Full,
                    openClose: true,
                },
                codeActionProvider: {
                    codeActionKinds: [...codeActions_1.CodeActionKinds],
                },
                completionProvider: {
                    triggerCharacters: ['.', '{{ ', '{% ', '<', '/', '[', '"', "'"],
                },
                documentLinkProvider: {
                    resolveProvider: false,
                    workDoneProgress: false,
                },
                executeCommandProvider: {
                    commands: [...commands_1.Commands],
                },
                hoverProvider: {
                    workDoneProgress: false,
                },
                workspace: {
                    fileOperations: {
                        didRename: fileOperationRegistrationOptions,
                        didCreate: fileOperationRegistrationOptions,
                        didDelete: fileOperationRegistrationOptions,
                    },
                },
            },
            serverInfo: {
                name: 'theme-language-server',
                version: version_1.VERSION,
            },
        };
        return result;
    });
    connection.onInitialized(() => {
        log(`[SERVER] Let's roll!`);
    });
    connection.onDidOpenTextDocument((params) => {
        const { uri, text, version } = params.textDocument;
        documentManager.open(uri, text, version);
        runChecks([uri]);
    });
    connection.onDidChangeTextDocument((params) => {
        const { uri, version } = params.textDocument;
        documentManager.change(uri, params.contentChanges[0].text, version);
        runChecks([uri]);
    });
    connection.onDidCloseTextDocument((params) => {
        const { uri } = params.textDocument;
        documentManager.close(uri);
        diagnosticsManager.clear(uri);
    });
    connection.onDocumentLinks(async (params) => {
        const { uri } = params.textDocument;
        const rootUri = await findRootURI(uri);
        return documentLinksProvider.documentLinks(uri, rootUri);
    });
    connection.onCodeAction(async (params) => {
        return codeActionsProvider.codeActions(params);
    });
    connection.onExecuteCommand(async (params) => {
        await executeCommandProvider.execute(params);
    });
    connection.onCompletion(async (params) => {
        return completionsProvider.completions(params);
    });
    connection.onHover(async (params) => {
        return hoverProvider.hover(params);
    });
    // These notifications could cause a MissingSnippet check to be invalidated
    //
    // It is not guaranteed that the file is or was opened when it was
    // created/renamed/deleted. If we're smart, we're going to re-lint for
    // every root affected. Unfortunately, we can't just use the debounced
    // call because we could run in a weird timing issue where didOpen
    // happens after the didRename and causes the 'lastArgs' to skip over the
    // ones we were after.
    //
    // So we're using runChecks.force for that.
    connection.onNotification(vscode_languageserver_1.DidCreateFilesNotification.type, (params) => {
        const triggerUris = params.files.map((fileCreate) => fileCreate.uri);
        runChecks.force(triggerUris);
    });
    connection.onNotification(vscode_languageserver_1.DidRenameFilesNotification.type, (params) => {
        const triggerUris = params.files.map((fileRename) => fileRename.newUri);
        runChecks.force(triggerUris);
    });
    connection.onNotification(vscode_languageserver_1.DidDeleteFilesNotification.type, (params) => {
        const triggerUris = params.files.map((fileDelete) => fileDelete.uri);
        runChecks.force(triggerUris);
    });
    connection.listen();
}
exports.startServer = startServer;
//# sourceMappingURL=startServer.js.map