import { CompletionItem, Position } from 'vscode-languageserver-protocol';
import { CompletionsProvider } from '../completions';
import { expect } from 'vitest';
import { AsyncExpectationResult, MatcherState, RawMatcherFn } from '@vitest/expect';

interface CustomMatchers<R = unknown> {
  /**
   * @param source {string} - the source to complete
   * @param completionItem {CompletionItem[] | string[]} - the list of completion items or labels
   *
   * @example
   * expect(provider).to.complete("{% end", ["endcomment"])
   */
  complete(source: string, completionItem: Partial<CompletionItem>[] | string[]): Promise<void>;
}

declare module 'vitest' {
  interface Assertion<T = any> extends CustomMatchers<T> {}
  interface AsymmetricMatchersContaining extends CustomMatchers {}
}

export const complete: RawMatcherFn<MatcherState> = async function (
  this: MatcherState,
  provider: CompletionsProvider,
  context: string,
  expected: any[],
): AsyncExpectationResult {
  const { isNot, equals, utils } = this;
  const source = createSourceCode(context);
  const completionParams = createCompletionParams(context);
  provider.documentManager.open(completionParams.textDocument.uri, source, 1);
  const result = await provider.completions(completionParams);

  return {
    pass:
      result.length === expected.length &&
      expected.every((expectation, i) =>
        equals(
          result[i],
          typeof expectation === 'string'
            ? expect.objectContaining({
                label: expectation,
              })
            : expectation,
        ),
      ),
    message: () =>
      `expected hover to${isNot ? ' not' : ''} match value ${utils.printExpected(
        expected,
      )}\ncontext:\n${context}`,
    actual: result,
    expected: expected,
  };
};

function createSourceCode(context: string) {
  const regex = new RegExp('█', 'g');
  return context.replace(regex, '');
}

function createCompletionParams(context: string) {
  const position = createPosition(context);
  const textDocument = { uri: '/path/to/file.liquid' };

  return {
    position,
    textDocument,
  };
}

function createPosition(context: string): Position {
  const index = context.indexOf('█');
  const lines = context.substring(0, index === -1 ? context.length : index).split('\n');
  const line = lines.length - 1;
  const character = lines[line].length;

  return { line, character };
}
